@@ -1,5 +1,5 @@
 /*
-Copyright 2018 The Knative Authors.
+Copyright 2018 The Knative Authors
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@ import (
 	"context"
 	"fmt"
 	"testing"
+	"time"
 
 	// Inject the fake informers that this controller needs.
 	servingclient "knative.dev/serving/pkg/client/injection/client/fake"
@@ -30,6 +31,7 @@ import (
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/apimachinery/pkg/util/clock"
 	clientgotesting "k8s.io/client-go/testing"
 	routereconciler "knative.dev/serving/pkg/client/injection/reconciler/serving/v1/route"
 
@@ -38,16 +40,20 @@ import (
 	"knative.dev/pkg/kmeta"
 	"knative.dev/pkg/logging"
 	"knative.dev/pkg/ptr"
+	"knative.dev/pkg/system"
+	cfgmap "knative.dev/serving/pkg/apis/config"
 	v1 "knative.dev/serving/pkg/apis/serving/v1"
+	autoscalercfg "knative.dev/serving/pkg/autoscaler/config"
 
 	. "knative.dev/pkg/reconciler/testing"
 	. "knative.dev/serving/pkg/reconciler/testing/v1"
 	. "knative.dev/serving/pkg/testing/v1"
 )
 
-// This is heavily based on the way the OpenShift Ingress controller tests its reconciliation method.
-func TestReconcile(t *testing.T) {
+func TestV2Reconcile(t *testing.T) {
 	now := metav1.Now()
+	fakeTime := now.Time
+	clock := clock.NewFakePassiveClock(fakeTime)
 
 	table := TableTest{{
 		Name: "bad workqueue key",
@@ -66,9 +72,9 @@ func TestReconcile(t *testing.T) {
 		},
 		WantPatches: []clientgotesting.PatchActionImpl{
 			patchAddFinalizerAction("default", "first-reconcile"),
-			patchAddLabel("default", rev("default", "the-config").Name,
-				"serving.knative.dev/route", "first-reconcile"),
-			patchAddLabel("default", "the-config", "serving.knative.dev/route", "first-reconcile"),
+			patchAddRouteAndServingStateLabel(
+				"default", rev("default", "the-config").Name, "first-reconcile", now.Time),
+			patchAddRouteAnn("default", "the-config", "first-reconcile"),
 		},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "FinalizerUpdate", "Updated %q finalizers", "first-reconcile"),
@@ -84,10 +90,10 @@ func TestReconcile(t *testing.T) {
 		},
 		WantPatches: []clientgotesting.PatchActionImpl{
 			patchAddFinalizerAction("default", "pinned-revision"),
-			patchAddLabel("default", "the-revision",
-				"serving.knative.dev/route", "pinned-revision"),
-			patchAddLabel("default", "the-config",
-				"serving.knative.dev/route", "pinned-revision"),
+			patchAddRouteAndServingStateLabel(
+				"default", "the-revision", "pinned-revision", now.Time),
+			patchAddRouteAnn("default", "the-config",
+				"pinned-revision"),
 		},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "FinalizerUpdate", "Updated %q finalizers", "pinned-revision"),
@@ -98,9 +104,11 @@ func TestReconcile(t *testing.T) {
 		Objects: []runtime.Object{
 			simpleRunLatest("default", "steady-state", "the-config", WithRouteFinalizer),
 			simpleConfig("default", "the-config",
-				WithConfigLabel("serving.knative.dev/route", "steady-state")),
+				WithConfigAnn("serving.knative.dev/routes", "steady-state")),
 			rev("default", "the-config",
-				WithRevisionLabel("serving.knative.dev/route", "steady-state")),
+				WithRevisionAnn("serving.knative.dev/routes", "steady-state"),
+				WithRoutingState(v1.RoutingStateActive, clock),
+				WithRoutingStateModified(now.Time)),
 		},
 		Key: "default/steady-state",
 	}, {
@@ -112,10 +120,10 @@ func TestReconcile(t *testing.T) {
 		},
 		WantPatches: []clientgotesting.PatchActionImpl{
 			patchAddFinalizerAction("default", "no-ready-revision"),
-			patchAddLabel("default", rev("default", "the-config").Name,
-				"serving.knative.dev/route", "no-ready-revision"),
-			patchAddLabel("default", "the-config",
-				"serving.knative.dev/route", "no-ready-revision"),
+			patchAddRouteAndServingStateLabel(
+				"default", rev("default", "the-config").Name, "no-ready-revision", now.Time),
+			patchAddRouteAnn("default", "the-config",
+				"no-ready-revision"),
 		},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "FinalizerUpdate", "Updated %q finalizers", "no-ready-revision"),
@@ -127,21 +135,22 @@ func TestReconcile(t *testing.T) {
 			simpleRunLatest("default", "transitioning-route", "old", WithRouteFinalizer,
 				WithSpecTraffic(configTraffic("new"))),
 			simpleConfig("default", "old",
-				WithConfigLabel("serving.knative.dev/route", "transitioning-route")),
+				WithConfigAnn("serving.knative.dev/routes", "transitioning-route")),
 			rev("default", "old",
-				WithRevisionLabel("serving.knative.dev/route", "transitioning-route")),
+				WithRevisionAnn("serving.knative.dev/routes", "transitioning-route"),
+				WithRoutingState(v1.RoutingStateActive, clock)),
 			simpleConfig("default", "new"),
 			rev("default", "new"),
 		},
 		WantPatches: []clientgotesting.PatchActionImpl{
-			patchAddLabel("default", rev("default", "new").Name,
-				"serving.knative.dev/route", "transitioning-route"),
-			patchAddLabel("default", "new",
-				"serving.knative.dev/route", "transitioning-route"),
+			patchAddRouteAndServingStateLabel(
+				"default", rev("default", "new").Name, "transitioning-route", now.Time),
+			patchAddRouteAnn("default", "new",
+				"transitioning-route"),
 		},
 		Key: "default/transitioning-route",
 	}, {
-		Name: "failure adding label (revision)",
+		Name: "failure adding annotation (revision)",
 		// Induce a failure during patching
 		WantErr: true,
 		WithReactors: []clientgotesting.ReactionFunc{
@@ -153,16 +162,16 @@ func TestReconcile(t *testing.T) {
 			rev("default", "the-config"),
 		},
 		WantPatches: []clientgotesting.PatchActionImpl{
-			patchAddLabel("default", rev("default", "the-config").Name,
-				"serving.knative.dev/route", "add-label-failure"),
+			patchAddRouteAndServingStateLabel(
+				"default", rev("default", "the-config").Name, "add-label-failure", now.Time),
 		},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeWarning, "InternalError",
-				`failed to add route label to /, Kind= "the-config-dbnfd": inducing failure for patch revisions`),
+				`failed to add route annotation to Namespace=default Name="the-config-dbnfd": inducing failure for patch revisions`),
 		},
 		Key: "default/add-label-failure",
 	}, {
-		Name: "failure adding label (configuration)",
+		Name: "failure adding annotation (configuration)",
 		// Induce a failure during patching
 		WantErr: true,
 		WithReactors: []clientgotesting.ReactionFunc{
@@ -172,49 +181,51 @@ func TestReconcile(t *testing.T) {
 			simpleRunLatest("default", "add-label-failure", "the-config", WithRouteFinalizer),
 			simpleConfig("default", "the-config"),
 			rev("default", "the-config",
-				WithRevisionLabel("serving.knative.dev/route", "add-label-failure")),
+				WithRevisionAnn("serving.knative.dev/routes", "add-label-failure"),
+				WithRoutingState(v1.RoutingStateActive, clock),
+				WithRoutingStateModified(now.Time)),
 		},
 		WantPatches: []clientgotesting.PatchActionImpl{
-			patchAddLabel("default", "the-config", "serving.knative.dev/route", "add-label-failure"),
+			patchAddRouteAnn("default", "the-config", "add-label-failure"),
 		},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeWarning, "InternalError",
-				`failed to add route label to /, Kind= "the-config": inducing failure for patch configurations`),
+				`failed to add route annotation to Namespace=default Name="the-config": inducing failure for patch configurations`),
 		},
 		Key: "default/add-label-failure",
 	}, {
-		Name:    "label config with incorrect label",
-		WantErr: true,
+		Name: "delete route existing ann",
 		Objects: []runtime.Object{
-			simpleRunLatest("default", "the-route", "the-config", WithRouteFinalizer),
+			simpleRunLatest("default", "delete-route", "the-config", WithRouteFinalizer, WithRouteDeletionTimestamp(&now)),
 			simpleConfig("default", "the-config",
-				WithConfigLabel("serving.knative.dev/route", "another-route")),
-			rev("default", "the-config",
-				WithRevisionLabel("serving.knative.dev/route", "another-route")),
+				WithConfigAnn("serving.knative.dev/routes", "delete-route,another-route")),
+		},
+		WantPatches: []clientgotesting.PatchActionImpl{
+			patchAddRouteAnn("default", "the-config", "another-route"),
+			patchRemoveFinalizerAction("default", "delete-route"),
 		},
 		WantEvents: []string{
-			Eventf(corev1.EventTypeWarning, "InternalError",
-				`/, Kind= "the-config-dbnfd" is already in use by "another-route", and cannot be used by "the-route"`),
+			Eventf(corev1.EventTypeNormal, "FinalizerUpdate", `Updated "delete-route" finalizers`),
 		},
-		Key: "default/the-route",
+		Key: "default/delete-route",
 	}, {
 		Name: "change configurations",
 		Objects: []runtime.Object{
 			simpleRunLatest("default", "config-change", "new-config", WithRouteFinalizer),
 			simpleConfig("default", "old-config",
-				WithConfigLabel("serving.knative.dev/route", "config-change")),
+				WithConfigAnn("serving.knative.dev/routes", "config-change")),
 			rev("default", "old-config",
-				WithRevisionLabel("serving.knative.dev/route", "config-change")),
+				WithRevisionAnn("serving.knative.dev/routes", "config-change"),
+				WithRoutingState(v1.RoutingStateActive, clock)),
 			simpleConfig("default", "new-config"),
 			rev("default", "new-config"),
 		},
 		WantPatches: []clientgotesting.PatchActionImpl{
-			patchRemoveLabel("default", rev("default", "old-config").Name,
-				"serving.knative.dev/route"),
-			patchAddLabel("default", rev("default", "new-config").Name,
-				"serving.knative.dev/route", "config-change"),
-			patchRemoveLabel("default", "old-config", "serving.knative.dev/route"),
-			patchAddLabel("default", "new-config", "serving.knative.dev/route", "config-change"),
+			patchRemoveRouteAndServingStateLabel("default", rev("default", "old-config").Name, now.Time),
+			patchRemoveRouteAnn("default", "old-config"),
+			patchAddRouteAndServingStateLabel(
+				"default", rev("default", "new-config").Name, "config-change", now.Time),
+			patchAddRouteAnn("default", "new-config", "config-change"),
 		},
 		Key: "default/config-change",
 	}, {
@@ -223,15 +234,16 @@ func TestReconcile(t *testing.T) {
 			simpleRunLatest("default", "config-update", "the-config", WithRouteFinalizer),
 			simpleConfig("default", "the-config",
 				WithLatestCreated("the-config-ecoge"),
-				WithConfigLabel("serving.knative.dev/route", "config-update")),
+				WithConfigAnn("serving.knative.dev/routes", "config-update")),
 			rev("default", "the-config",
-				WithRevisionLabel("serving.knative.dev/route", "config-update")),
+				WithRevisionAnn("serving.knative.dev/routes", "config-update"),
+				WithRoutingState(v1.RoutingStateActive, clock)),
 			rev("default", "the-config",
 				WithRevName("the-config-ecoge")),
 		},
 		WantPatches: []clientgotesting.PatchActionImpl{
-			patchAddLabel("default", "the-config-ecoge",
-				"serving.knative.dev/route", "config-update"),
+			patchAddRouteAndServingStateLabel(
+				"default", "the-config-ecoge", "config-update", now.Time),
 		},
 		Key: "default/config-update",
 	}, {
@@ -239,16 +251,35 @@ func TestReconcile(t *testing.T) {
 		Objects: []runtime.Object{
 			simpleRunLatest("default", "delete-route", "the-config", WithRouteFinalizer, WithRouteDeletionTimestamp(&now)),
 			simpleConfig("default", "the-config",
-				WithConfigLabel("serving.knative.dev/route", "delete-route")),
+				WithConfigAnn("serving.knative.dev/routes", "delete-route")),
 		},
 		WantPatches: []clientgotesting.PatchActionImpl{
-			patchRemoveLabel("default", "the-config", "serving.knative.dev/route"),
+			patchRemoveRouteAnn("default", "the-config"),
 			patchRemoveFinalizerAction("default", "delete-route"),
 		},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeNormal, "FinalizerUpdate", `Updated "delete-route" finalizers`),
 		},
 		Key: "default/delete-route",
+	}, {
+		Name:    "delete route failure",
+		WantErr: true,
+		WithReactors: []clientgotesting.ReactionFunc{
+			InduceFailure("patch", "configurations"),
+		},
+		Objects: []runtime.Object{
+			simpleRunLatest("default", "delete-route", "the-config", WithRouteFinalizer, WithRouteDeletionTimestamp(&now)),
+			simpleConfig("default", "the-config",
+				WithConfigAnn("serving.knative.dev/routes", "delete-route")),
+		},
+		WantPatches: []clientgotesting.PatchActionImpl{
+			patchRemoveRouteAnn("default", "the-config"),
+		},
+		WantEvents: []string{
+			Eventf(corev1.EventTypeWarning, "InternalError",
+				`failed to remove route annotation to /, Kind= "the-config": inducing failure for patch configurations`),
+		},
+		Key: "default/delete-route",
 	}, {
 		Name: "failure while removing a cfg annotation should return an error",
 		// Induce a failure during patching
@@ -259,19 +290,21 @@ func TestReconcile(t *testing.T) {
 		Objects: []runtime.Object{
 			simpleRunLatest("default", "delete-label-failure", "new-config", WithRouteFinalizer),
 			simpleConfig("default", "old-config",
-				WithConfigLabel("serving.knative.dev/route", "delete-label-failure")),
+				WithConfigAnn("serving.knative.dev/routes", "delete-label-failure")),
 			simpleConfig("default", "new-config",
-				WithConfigLabel("serving.knative.dev/route", "delete-label-failure")),
+				WithConfigAnn("serving.knative.dev/routes", "delete-label-failure")),
 			rev("default", "new-config",
-				WithRevisionLabel("serving.knative.dev/route", "delete-label-failure")),
+				WithRevisionAnn("serving.knative.dev/routes", "delete-label-failure"),
+				WithRoutingState(v1.RoutingStateActive, clock),
+				WithRoutingStateModified(now.Time)),
 			rev("default", "old-config"),
 		},
 		WantPatches: []clientgotesting.PatchActionImpl{
-			patchRemoveLabel("default", "old-config", "serving.knative.dev/route"),
+			patchRemoveRouteAnn("default", "old-config"),
 		},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeWarning, "InternalError",
-				`failed to remove route label to /, Kind= "old-config": inducing failure for patch configurations`),
+				`failed to remove route annotation to /, Kind= "old-config": inducing failure for patch configurations`),
 		},
 		Key: "default/delete-label-failure",
 	}, {
@@ -284,30 +317,34 @@ func TestReconcile(t *testing.T) {
 		Objects: []runtime.Object{
 			simpleRunLatest("default", "delete-label-failure", "new-config", WithRouteFinalizer),
 			simpleConfig("default", "old-config",
-				WithConfigLabel("serving.knative.dev/route", "delete-label-failure")),
+				WithConfigAnn("serving.knative.dev/routes", "delete-label-failure")),
 			simpleConfig("default", "new-config",
-				WithConfigLabel("serving.knative.dev/route", "delete-label-failure")),
+				WithConfigAnn("serving.knative.dev/routes", "delete-label-failure")),
 			rev("default", "new-config",
-				WithRevisionLabel("serving.knative.dev/route", "delete-label-failure")),
+				WithRevisionAnn("serving.knative.dev/routes", "delete-label-failure")),
 			rev("default", "old-config",
-				WithRevisionLabel("serving.knative.dev/route", "delete-label-failure")),
+				WithRoutingState(v1.RoutingStateActive, clock),
+				WithRevisionAnn("serving.knative.dev/routes", "delete-label-failure")),
 		},
 		WantPatches: []clientgotesting.PatchActionImpl{
-			patchRemoveLabel("default", rev("default", "old-config").Name,
-				"serving.knative.dev/route"),
+			patchRemoveRouteAndServingStateLabel("default", rev("default", "old-config").Name, now.Time),
 		},
 		WantEvents: []string{
 			Eventf(corev1.EventTypeWarning, "InternalError",
-				`failed to remove route label to /, Kind= "old-config-dbnfd": inducing failure for patch revisions`),
+				`failed to remove route annotation to /, Kind= "old-config-dbnfd": inducing failure for patch revisions`),
 		},
 		Key: "default/delete-label-failure",
 	}}
 
 	table.Test(t, MakeFactory(func(ctx context.Context, listers *Listers, cmw configmap.Watcher) controller.Reconciler {
+		client := servingclient.Get(ctx)
+		cLister := listers.GetConfigurationLister()
+		cIndexer := listers.IndexerFor(&v1.Configuration{})
+		rLister := listers.GetRevisionLister()
+		rIndexer := listers.IndexerFor(&v1.Revision{})
 		r := &Reconciler{
-			client:              servingclient.Get(ctx),
-			configurationLister: listers.GetConfigurationLister(),
-			revisionLister:      listers.GetRevisionLister(),
+			caccV2: newConfigurationAccessor(client, &NullTracker{}, cLister, cIndexer, clock),
+			raccV2: newRevisionAccessor(client, &NullTracker{}, rLister, rIndexer, clock),
 		}
 
 		return routereconciler.NewReconciler(ctx, logging.FromContext(ctx), servingclient.Get(ctx),
@@ -333,7 +370,8 @@ func revTraffic(name string, latest bool) v1.TrafficTarget {
 
 func routeWithTraffic(namespace, name string, spec, status v1.TrafficTarget, opts ...RouteOption) *v1.Route {
 	return Route(namespace, name,
-		append([]RouteOption{WithSpecTraffic(spec), WithStatusTraffic(status)}, opts...)...)
+		append([]RouteOption{WithSpecTraffic(spec), WithStatusTraffic(status), WithInitRouteConditions,
+			MarkTrafficAssigned, MarkCertificateReady, MarkIngressReady, WithRouteObservedGeneration}, opts...)...)
 }
 
 func simpleRunLatest(namespace, name, config string, opts ...RouteOption) *v1.Route {
@@ -384,52 +422,92 @@ func rev(namespace, name string, opts ...RevisionOption) *v1.Revision {
 	return rev
 }
 
-func patchRemoveLabel(namespace, name, key string) clientgotesting.PatchActionImpl {
-	action := clientgotesting.PatchActionImpl{}
-	action.Name = name
-	action.Namespace = namespace
+func patchRemoveRouteAnn(namespace, name string) clientgotesting.PatchActionImpl {
+	return patchAddRouteAnn(namespace, name, "null")
+}
 
-	patch := fmt.Sprintf(`{"metadata":{"labels":{%q:null}}}`, key)
+func patchAddRouteAnn(namespace, name, value string) clientgotesting.PatchActionImpl {
+	action := clientgotesting.PatchActionImpl{
+		Name:       name,
+		ActionImpl: clientgotesting.ActionImpl{Namespace: namespace},
+	}
 
-	action.Patch = []byte(patch)
+	// Note: the raw json `"key": null` removes a value, whereas an actual value
+	// called "null" would need quotes to parse as a string `"key":"null"`.
+	if value != "null" {
+		value = `"` + value + `"`
+	}
+
+	action.Patch = []byte(fmt.Sprintf(`{"metadata":{"annotations":{"serving.knative.dev/routes":%s}}}`, value))
 	return action
 }
 
-func patchAddLabel(namespace, name, key, value string) clientgotesting.PatchActionImpl {
-	action := clientgotesting.PatchActionImpl{}
-	action.Name = name
-	action.Namespace = namespace
+func patchRemoveRouteAndServingStateLabel(namespace, name string, now time.Time) clientgotesting.PatchActionImpl {
+	return patchAddRouteAndServingStateLabel(namespace, name, "null", now)
+}
 
-	patch := fmt.Sprintf(`{"metadata":{"labels":{%q:%q}}}`, key, value)
+func patchAddRouteAndServingStateLabel(namespace, name, routeName string, now time.Time) clientgotesting.PatchActionImpl {
+	action := clientgotesting.PatchActionImpl{
+		Name:       name,
+		ActionImpl: clientgotesting.ActionImpl{Namespace: namespace},
+	}
 
-	action.Patch = []byte(patch)
+	// Note: the raw json `"key": null` removes a value, whereas an actual value
+	// called "null" would need quotes to parse as a string `"key":"null"`.
+	state := string(v1.RoutingStateReserve)
+	if routeName != "null" {
+		state = string(v1.RoutingStateActive)
+		routeName = `"` + routeName + `"`
+	}
+
+	action.Patch = []byte(fmt.Sprintf(
+		`{"metadata":{"annotations":{"serving.knative.dev/routes":%s,`+
+			`"serving.knative.dev/routingStateModified":%q},`+
+			`"labels":{"serving.knative.dev/routingState":%q}}}`, routeName, now.UTC().Format(time.RFC3339), state))
 	return action
 }
 
 func patchAddFinalizerAction(namespace, name string) clientgotesting.PatchActionImpl {
-	resource := v1.Resource("routes")
-	finalizer := resource.String()
-
-	action := clientgotesting.PatchActionImpl{}
-	action.Name = name
-	action.Namespace = namespace
-	patch := fmt.Sprintf(`{"metadata":{"finalizers":[%q],"resourceVersion":""}}`, finalizer)
-	action.Patch = []byte(patch)
-	return action
+	p := fmt.Sprintf(`{"metadata":{"finalizers":[%q],"resourceVersion":""}}`, v1.Resource("routes").String())
+	return clientgotesting.PatchActionImpl{
+		Name:       name,
+		ActionImpl: clientgotesting.ActionImpl{Namespace: namespace},
+		Patch:      []byte(p),
+	}
 }
 
 func patchRemoveFinalizerAction(namespace, name string) clientgotesting.PatchActionImpl {
-	action := clientgotesting.PatchActionImpl{}
-	action.Name = name
-	action.Namespace = namespace
-	action.Patch = []byte(`{"metadata":{"finalizers":[],"resourceVersion":""}}`)
-	return action
+	return clientgotesting.PatchActionImpl{
+		Name:       name,
+		ActionImpl: clientgotesting.ActionImpl{Namespace: namespace},
+		Patch:      []byte(`{"metadata":{"finalizers":[],"resourceVersion":""}}`),
+	}
 }
 
 func TestNew(t *testing.T) {
 	ctx, _ := SetupFakeContext(t)
 
-	c := NewController(ctx, configmap.NewStaticWatcher())
+	configMapWatcher := configmap.NewStaticWatcher(&corev1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      cfgmap.FeaturesConfigName,
+			Namespace: system.Namespace(),
+		},
+		Data: map[string]string{},
+	}, &corev1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      cfgmap.DefaultsConfigName,
+			Namespace: system.Namespace(),
+		},
+		Data: map[string]string{},
+	}, &corev1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      autoscalercfg.ConfigName,
+			Namespace: system.Namespace(),
+		},
+		Data: map[string]string{},
+	})
+
+	c := NewController(ctx, configMapWatcher)
 
 	if c == nil {
 		t.Fatal("Expected NewController to return a non-nil value")
