@@ -80,13 +80,12 @@ var (
 		StartupProbe: &corev1.Probe{
 			Handler: corev1.Handler{
 				Exec: &corev1.ExecAction{
-					Command: []string{"/ko-app/queue", "-probe-timeout", "1h34m38s"},
+					Command: []string{"/ko-app/queue", "-probe-timeout", "0s"},
 				},
 			},
 			PeriodSeconds:    1,
 			FailureThreshold: 1,
 			SuccessThreshold: 1,
-			TimeoutSeconds:   5678,
 		},
 		ReadinessProbe: &corev1.Probe{
 			Handler: corev1.Handler{
@@ -152,9 +151,6 @@ var (
 		}, {
 			Name:  "TRACING_CONFIG_ZIPKIN_ENDPOINT",
 			Value: "",
-		}, {
-			Name:  "TRACING_CONFIG_STACKDRIVER_PROJECT_ID",
-			Value: "",
 		}, {
 			Name:  "TRACING_CONFIG_DEBUG",
 			Value: "false",
@@ -182,6 +178,12 @@ var (
 		}, {
 			Name:  "METRICS_COLLECTOR_ADDRESS",
 			Value: "",
+		}, {
+			Name:  "CONCURRENCY_STATE_ENDPOINT",
+			Value: "",
+		}, {
+			Name:  "ENABLE_HTTP2_AUTO_DETECTION",
+			Value: "false",
 		}},
 	}
 
